
#include <stdio.h>
#include "inttypes.h"
#include "Date.h"



uint8_t Date_Month_Offset[DATE_MONTH]={0,3,3,6,1,4,6,2,5,0,3,5};
uint8_t Date_Month[DATE_MONTH]={31,28,31,30,31,30,31,31,30,31,30,31};
uint8_t Date_Month_CalWeek_Offset[DATE_MONTH]={0,4,8,12,17,21,25,30,34,39,43,47};

void Date_Print (date_t* p_date)
{


    printf("\t\t D: %2d.%2d.%4d WD: %1d CW: %2d LJ: %1d\n",p_date->day,
                                            p_date->month,
                                            p_date->year,
                                            p_date->week,
                                            p_date->calweek,
                                            p_date->leapjear);

}
void Date_Calc_Leapjear(date_t* p_date)
{
    if(((p_date->year%4==0) && (p_date->year%100!=0)) || (p_date->year%400==0))
    {
       p_date->leapjear=1;
    } 
    else
    {
       p_date->leapjear=0;
    }
}

uint8_t Date_Check_Date (date_t* p_date)
{
    uint8_t day=0;
    //input check
	
    //check month
    if((p_date->month>12)||(p_date->month<1))
    {
        return 0;
    }
    //check jear
    if(p_date->year>7999)
    {
        return 0;
    }
    //calculate leapyear

    Date_Calc_Leapjear(p_date);

    //check day

    day=Date_Month[p_date->month-1];
    if((p_date->month==2)&&p_date->leapjear)
        day++;

    if((p_date->day>day)||(p_date->day<1))
    {
        return 0;
    }

    return 1;
}

void Date_Calc_Week (date_t* p_date)
{

    uint8_t week=0;
    uint8_t century=0;
    uint8_t yearone=0;

   
    //weekday calculation

    //day
    week=p_date->day;

    //month
    week+=Date_Month_Offset[p_date->month-1];

    //
    Date_Calc_Leapjear(p_date);

    //leapjear
    if((p_date->month<3)&&p_date->leapjear)
        week--;

    //year one
    yearone=p_date->year%100;
    week+=((yearone+yearone/4)%7);

    //century
    century=p_date->year/100;
    week+=((3-(century%4))*2);

	//offset corr (0=>Mo...6=>So)
	week+=6;

    //end
    week%=7;
    p_date->week=week;
}
//returns 1 if the last day in a year is in a 53 calendarweek
uint8_t Date_Calc_If_53Week_End (date_t* p_date)
{
    date_t date_begin, date_end;
    //calc 01.01.x
    date_begin.day=1;
    date_begin.month=1;
    date_begin.year=p_date->year;
    Date_Calc_Week(&date_begin);
    //calc 31.12.x
    date_end.day=31;
    date_end.month=12;
    date_end.year=p_date->year;
    Date_Calc_Week(&date_end);

    if((date_begin.week==3)||(date_end.week==3))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
//returns 1 if the first day in a year is in a 53 calendarweek
uint8_t Date_Calc_If_53Week_Begin (date_t* p_date)
{
    date_t date_begin;
    //calc 01.01.x
    date_begin.day=1;
    date_begin.month=1;
    date_begin.year=p_date->year;
    Date_Calc_Week(&date_begin);

    if(date_begin.week==4)
    {
        return 1;
    }
    else
    {
        //if this year %4=1 the year before was %4=0  AND this date begins with saturday AND year %400 is not 101
        if(((p_date->year&0x3)==1)&&(date_begin.week==5)&&((p_date->year%400)!=101))
        {
            return 1;
        }
        else
        {
            return 0;
        }
    }
}
//calweek calc with prints
void Date_Calc_Calweek(date_t* p_date)
{
	uint8_t calweek=0;

	date_t date_begin;
	//calc 04.01.x
    date_begin.day=4;
    date_begin.month=1;
    date_begin.year=p_date->year;
    Date_Calc_Week(&date_begin);

	//get day "offset"
	calweek=date_begin.week;

	//add month offset
	calweek+=Date_Month_Offset[p_date->month-1];

	//
	calweek+=p_date->day;

	//leapjear
    Date_Calc_Leapjear(p_date);
    if((p_date->month>2)&&p_date->leapjear)
        calweek++;

    calweek=(calweek+3)/7;
    
    //add calweek offset
    calweek+=Date_Month_CalWeek_Offset[p_date->month-1];
    
    //Date_Print(p_date);

    if(calweek==0)
    {
        if(Date_Calc_If_53Week_Begin(p_date))
        {
            calweek=53;
        }
        else
        {
            calweek=52;
        }
    }
    else if(calweek==53)
    {
        if(!Date_Calc_If_53Week_End(p_date))
        {
            calweek=1;
        }
    }
    p_date->calweek=calweek;
}
//returns days in Month
uint8_t Date_Calc_MaxDay (date_t* p_date)
{
    uint8_t day=0;

    //calculate leapyear

    Date_Calc_Leapjear(p_date);

    //check day

    day=Date_Month[p_date->month-1];
    if((p_date->month==2)&&p_date->leapjear)
        day++;

    return day;
}


